@@ -1,4 +1,4 @@
-//! API to generate `.rs` files.
+//! # API to generate `.rs` files
 //!
 //! This API does not require `protoc` command present in `$PATH`.
 //!
@@ -15,13 +15,29 @@
 //! }
 //! ```
 //!
+//! And in `Cargo.toml`:
+//!
+//! ```toml
+//! [build-dependencies]
+//! protobuf-codegen-pure = "2"
+//! ```
+//!
 //! It is advisable that `protobuf-codegen-pure` build-dependecy version be the same as
 //! `protobuf` dependency.
 //!
-//! The alternative is to use `protoc-rust` crate.
+//! The alternative is to use [`protoc-rust`](https://docs.rs/protoc-rust/=2) crate
+//! which uses `protoc` command for parsing (so it uses the same parser
+//! Google is using in their protobuf implementations).
+//!
+//! # Version 2
+//!
+//! This is documentation for version 2 of the crate.
+//!
+//! In version 3, this API is moved to
+//! [`protobuf-codegen` crate](https://docs.rs/protobuf-codegen/%3E=3.0.0-alpha).
 
 #![deny(missing_docs)]
-#![deny(broken_intra_doc_links)]
+#![deny(rustdoc::broken_intra_doc_links)]
 
 extern crate protobuf;
 extern crate protobuf_codegen;
@@ -30,6 +46,7 @@ mod convert;
 
 use std::error::Error;
 use std::fmt;
+use std::fmt::Formatter;
 use std::fs;
 use std::io;
 use std::io::Read;
@@ -162,6 +179,15 @@ enum CodegenError {
     ConvertError(convert::ConvertError),
 }
 
+impl fmt::Display for CodegenError {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        match self {
+            CodegenError::ParserErrorWithLocation(_) => write!(f, "Parse error"),
+            CodegenError::ConvertError(_) => write!(f, "Could not typecheck parsed file"),
+        }
+    }
+}
+
 impl From<parser::ParserErrorWithLocation> for CodegenError {
     fn from(e: parser::ParserErrorWithLocation) -> Self {
         CodegenError::ParserErrorWithLocation(e)
@@ -182,7 +208,7 @@ struct WithFileError {
 
 impl fmt::Display for WithFileError {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "WithFileError")
+        write!(f, "Could not write to {}: {}", self.file, self.error)
     }
 }
 
