@@ -1,17 +1,16 @@
-use std::str;
+use std::f64;
 use std::fmt;
 use std::num::ParseIntError;
-use std::f64;
+use std::str;
 
 use model::*;
+use protobuf::text_format::lexer::StrLit;
+use protobuf::text_format::lexer::StrLitDecodeError;
 use protobuf_codegen::float;
-use str_lit::*;
-
 
 const FIRST_LINE: u32 = 1;
 const FIRST_COL: u32 = 1;
 
-
 /// Location in file
 #[derive(Copy, Clone, Debug, Eq, PartialEq)]
 pub struct Loc {
@@ -36,7 +35,6 @@ impl Loc {
     }
 }
 
-
 /// Basic information about parsing error.
 #[derive(Debug)]
 pub enum ParserError {
@@ -59,6 +57,9 @@ pub enum ParserError {
     StrLitDecodeError(StrLitDecodeError),
     GroupNameShouldStartWithUpperCase,
     MapFieldNotAllowed,
+    OneOfInGroup,
+    OneOfInOneOf,
+    OneOfInExtend,
 }
 
 #[derive(Debug)]
@@ -90,7 +91,6 @@ impl From<float::ProtobufFloatParseError> for ParserError {
 
 pub type ParserResult<T> = Result<T, ParserError>;
 
-
 trait ToU8 {
     fn to_u8(&self) -> ParserResult<u8>;
 }
@@ -173,7 +173,6 @@ impl U64Extensions for u64 {
     }
 }
 
-
 #[derive(Clone, Debug, PartialEq)]
 enum Token {
     Ident(String),
@@ -234,7 +233,8 @@ impl<'a> Lexer<'a> {
     }
 
     fn lookahead_char_is_in(&self, alphabet: &str) -> bool {
-        self.lookahead_char().map_or(false, |c| alphabet.contains(c))
+        self.lookahead_char()
+            .map_or(false, |c| alphabet.contains(c))
     }
 
     fn next_char_opt(&mut self) -> Option<char> {
@@ -269,9 +269,7 @@ impl<'a> Lexer<'a> {
         if self.skip_if_lookahead_is_str("/*") {
             let end = "*/";
             match self.rem_chars().find(end) {
-                None => {
-                    Err(ParserError::UnexpectedEof)
-                }
+                None => Err(ParserError::UnexpectedEof),
                 Some(len) => {
                     let new_pos = self.pos + len + end.len();
                     self.skip_to_pos(new_pos);
@@ -302,13 +300,14 @@ impl<'a> Lexer<'a> {
             self.skip_block_comment();
             if pos == self.pos {
                 // Did not advance
-                return Ok(())
+                return Ok(());
             }
         }
     }
 
     fn take_while<F>(&mut self, f: F) -> &'a str
-        where F : Fn(char) -> bool
+    where
+        F: Fn(char) -> bool,
     {
         let start = self.pos;
         while self.lookahead_char().map(&f) == Some(true) {
@@ -337,14 +336,15 @@ impl<'a> Lexer<'a> {
     }
 
     fn next_char_if<P>(&mut self, p: P) -> Option<char>
-        where P : FnOnce(char) -> bool
+    where
+        P: FnOnce(char) -> bool,
     {
         let mut clone = self.clone();
         match clone.next_char_opt() {
             Some(c) if p(c) => {
                 *self = clone;
                 Some(c)
-            },
+            }
             _ => None,
         }
     }
@@ -399,9 +399,7 @@ impl<'a> Lexer<'a> {
     }
 
     fn is_ascii_alphanumeric(c: char) -> bool {
-        (c >= 'A' && c <= 'Z')
-        || (c >= 'a' && c <= 'z')
-        || (c >= '0' && c <= '9')
+        (c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z') || (c >= '0' && c <= '9')
     }
 
     fn next_ident_part(&mut self) -> Option<char> {
@@ -427,19 +425,19 @@ impl<'a> Lexer<'a> {
     // Integer literals
 
     fn is_ascii_hexdigit(c: char) -> bool {
-        (c >= '0' && c <= '9')
-        || (c >= 'a' && c <= 'f')
-        || (c >= 'A' && c <= 'F')
+        (c >= '0' && c <= '9') || (c >= 'a' && c <= 'f') || (c >= 'A' && c <= 'F')
     }
 
     // hexLit     = "0" ( "x" | "X" ) hexDigit { hexDigit }
     fn next_hex_lit(&mut self) -> ParserResult<Option<u64>> {
-        Ok(if self.skip_if_lookahead_is_str("0x") || self.skip_if_lookahead_is_str("0X") {
-            let s = self.take_while(Lexer::is_ascii_hexdigit);
-            Some(u64::from_str_radix(s, 16)? as u64)
-        } else {
-            None
-        })
+        Ok(
+            if self.skip_if_lookahead_is_str("0x") || self.skip_if_lookahead_is_str("0X") {
+                let s = self.take_while(Lexer::is_ascii_hexdigit);
+                Some(u64::from_str_radix(s, 16)? as u64)
+            } else {
+                None
+            },
+        )
     }
 
     fn is_ascii_digit(c: char) -> bool {
@@ -544,7 +542,7 @@ impl<'a> Lexer<'a> {
                 self.next_exponent_opt()?;
             } else {
                 if self.next_exponent_opt()? == None {
-                    return Err(ParserError::IncorrectFloatLit)
+                    return Err(ParserError::IncorrectFloatLit);
                 }
             }
         }
@@ -623,7 +621,7 @@ impl<'a> Lexer<'a> {
             }
             self.next_char_expect_eq(q)?;
 
-            raw.push_str(&self.input[start + 1 .. self.pos - 1]);
+            raw.push_str(&self.input[start + 1..self.pos - 1]);
         }
         Ok(raw)
     }
@@ -638,8 +636,8 @@ impl<'a> Lexer<'a> {
 
     fn is_ascii_punctuation(c: char) -> bool {
         match c {
-            '.' | ',' | ':' | ';' | '/' | '\\' | '=' | '%' | '+' | '-' | '*' |
-            '<' | '>' | '(' | ')' | '{' | '}' | '[' | ']' => true,
+            '.' | ',' | ':' | ';' | '/' | '\\' | '=' | '%' | '+' | '-' | '*' | '<' | '>' | '('
+            | ')' | '{' | '}' | '[' | ']' => true,
             _ => false,
         }
     }
@@ -700,7 +698,6 @@ impl<'a> Lexer<'a> {
     }
 }
 
-
 #[derive(Clone)]
 pub struct Parser<'a> {
     lexer: Lexer<'a>,
@@ -720,73 +717,63 @@ enum MessageBodyParseMode {
 impl MessageBodyParseMode {
     fn label_allowed(&self, label: Rule) -> bool {
         match label {
-            Rule::Repeated => {
-                match *self {
-                    MessageBodyParseMode::MessageProto2 |
-                    MessageBodyParseMode::MessageProto3 |
-                    MessageBodyParseMode::ExtendProto2 |
-                    MessageBodyParseMode::ExtendProto3 => true,
-                    MessageBodyParseMode::Oneof => false,
-                }
-            }
-            Rule::Optional | Rule::Required => {
-                match *self {
-                    MessageBodyParseMode::MessageProto2 |
-                    MessageBodyParseMode::ExtendProto2 => true,
-                    MessageBodyParseMode::MessageProto3 |
-                    MessageBodyParseMode::ExtendProto3 |
-                    MessageBodyParseMode::Oneof => false,
-                }
-
-            }
+            Rule::Repeated => match *self {
+                MessageBodyParseMode::MessageProto2
+                | MessageBodyParseMode::MessageProto3
+                | MessageBodyParseMode::ExtendProto2
+                | MessageBodyParseMode::ExtendProto3 => true,
+                MessageBodyParseMode::Oneof => false,
+            },
+            Rule::Optional | Rule::Required => match *self {
+                MessageBodyParseMode::MessageProto2 | MessageBodyParseMode::ExtendProto2 => true,
+                MessageBodyParseMode::MessageProto3
+                | MessageBodyParseMode::ExtendProto3
+                | MessageBodyParseMode::Oneof => false,
+            },
         }
     }
 
     fn some_label_required(&self) -> bool {
         match *self {
-            MessageBodyParseMode::MessageProto2 |
-            MessageBodyParseMode::ExtendProto2 => true,
-            MessageBodyParseMode::MessageProto3 |
-            MessageBodyParseMode::ExtendProto3 |
-            MessageBodyParseMode::Oneof => false,
+            MessageBodyParseMode::MessageProto2 | MessageBodyParseMode::ExtendProto2 => true,
+            MessageBodyParseMode::MessageProto3
+            | MessageBodyParseMode::ExtendProto3
+            | MessageBodyParseMode::Oneof => false,
         }
     }
 
     fn map_allowed(&self) -> bool {
         match *self {
-            MessageBodyParseMode::MessageProto2 |
-            MessageBodyParseMode::MessageProto3 |
-            MessageBodyParseMode::ExtendProto2 |
-            MessageBodyParseMode::ExtendProto3 => true,
+            MessageBodyParseMode::MessageProto2
+            | MessageBodyParseMode::MessageProto3
+            | MessageBodyParseMode::ExtendProto2
+            | MessageBodyParseMode::ExtendProto3 => true,
             MessageBodyParseMode::Oneof => false,
         }
     }
 
     fn is_most_non_fields_allowed(&self) -> bool {
         match *self {
-            MessageBodyParseMode::MessageProto2 |
-            MessageBodyParseMode::MessageProto3 => true,
-            MessageBodyParseMode::ExtendProto2 |
-            MessageBodyParseMode::ExtendProto3 |
-            MessageBodyParseMode::Oneof => false,
+            MessageBodyParseMode::MessageProto2 | MessageBodyParseMode::MessageProto3 => true,
+            MessageBodyParseMode::ExtendProto2
+            | MessageBodyParseMode::ExtendProto3
+            | MessageBodyParseMode::Oneof => false,
         }
     }
 
     fn is_option_allowed(&self) -> bool {
         match *self {
-            MessageBodyParseMode::MessageProto2 |
-            MessageBodyParseMode::MessageProto3 |
-            MessageBodyParseMode::Oneof => true,
-            MessageBodyParseMode::ExtendProto2 |
-            MessageBodyParseMode::ExtendProto3 => false,
+            MessageBodyParseMode::MessageProto2
+            | MessageBodyParseMode::MessageProto3
+            | MessageBodyParseMode::Oneof => true,
+            MessageBodyParseMode::ExtendProto2 | MessageBodyParseMode::ExtendProto3 => false,
         }
     }
 }
 
 #[derive(Default)]
 pub struct MessageBody {
-    pub fields: Vec<Field>,
-    pub oneofs: Vec<OneOf>,
+    pub fields: Vec<FieldOrOneOf>,
     pub reserved_nums: Vec<FieldNumberRange>,
     pub reserved_names: Vec<String>,
     pub messages: Vec<Message>,
@@ -850,7 +837,10 @@ impl<'a> Parser<'a> {
 
     fn next(&mut self) -> ParserResult<Option<Token>> {
         self.lookahead()?;
-        Ok(self.next_token.take().map(|TokenWithLocation { token, .. }| token))
+        Ok(self
+            .next_token
+            .take()
+            .map(|TokenWithLocation { token, .. }| token))
     }
 
     fn next_some(&mut self) -> ParserResult<Token> {
@@ -862,7 +852,8 @@ impl<'a> Parser<'a> {
 
     /// Can be called only after lookahead, otherwise it's error
     fn advance(&mut self) -> ParserResult<Token> {
-        self.next_token.take()
+        self.next_token
+            .take()
             .map(|TokenWithLocation { token, .. }| token)
             .ok_or(ParserError::InternalError)
     }
@@ -873,16 +864,15 @@ impl<'a> Parser<'a> {
     }
 
     fn next_token_if_map<P, R>(&mut self, p: P) -> ParserResult<Option<R>>
-        where P : FnOnce(&Token) -> Option<R>
+    where
+        P: FnOnce(&Token) -> Option<R>,
     {
         self.lookahead()?;
         let v = match self.next_token {
-            Some(ref token) => {
-                match p(&token.token) {
-                    Some(v) => v,
-                    None => return Ok(None),
-                }
-            }
+            Some(ref token) => match p(&token.token) {
+                Some(v) => v,
+                None => return Ok(None),
+            },
             _ => return Ok(None),
         };
         self.next_token = None;
@@ -890,7 +880,8 @@ impl<'a> Parser<'a> {
     }
 
     fn next_token_check_map<P, R>(&mut self, p: P) -> ParserResult<R>
-        where P : FnOnce(&Token) -> ParserResult<R>
+    where
+        P: FnOnce(&Token) -> ParserResult<R>,
     {
         self.lookahead()?;
         let r = match self.next_token {
@@ -902,7 +893,8 @@ impl<'a> Parser<'a> {
     }
 
     fn next_token_if<P>(&mut self, p: P) -> ParserResult<Option<Token>>
-        where P : FnOnce(&Token) -> bool
+    where
+        P: FnOnce(&Token) -> bool,
     {
         self.next_token_if_map(|token| if p(token) { Some(token.clone()) } else { None })
     }
@@ -963,20 +955,16 @@ impl<'a> Parser<'a> {
     // Protobuf grammar
 
     fn next_ident(&mut self) -> ParserResult<String> {
-        self.next_token_check_map(|token| {
-            match token {
-                &Token::Ident(ref ident) => Ok(ident.clone()),
-                _ => Err(ParserError::ExpectIdent),
-            }
+        self.next_token_check_map(|token| match token {
+            &Token::Ident(ref ident) => Ok(ident.clone()),
+            _ => Err(ParserError::ExpectIdent),
         })
     }
 
     fn next_str_lit(&mut self) -> ParserResult<StrLit> {
-        self.next_token_check_map(|token| {
-            match token {
-                &Token::StrLit(ref str_lit) => Ok(str_lit.clone()),
-                _ => Err(ParserError::IncorrectInput),
-            }
+        self.next_token_check_map(|token| match token {
+            &Token::StrLit(ref str_lit) => Ok(str_lit.clone()),
+            _ => Err(ParserError::IncorrectInput),
         })
     }
 
@@ -1067,35 +1055,30 @@ impl<'a> Parser<'a> {
             }
         }
 
-        if let Some(r) = self.next_token_if_map(|token| {
-                match token {
-                    &Token::StrLit(ref s) => Some(ProtobufConstant::String(s.clone())),
-                    _ => None,
-                }
-            })?
-        {
+        if let Some(r) = self.next_token_if_map(|token| match token {
+            &Token::StrLit(ref s) => Some(ProtobufConstant::String(s.clone())),
+            _ => None,
+        })? {
             return Ok(r);
         }
 
         match self.lookahead_some()? {
             &Token::IntLit(..) | &Token::FloatLit(..) => {
                 return self.next_num_lit()?.to_option_value(true);
-            },
+            }
             &Token::Ident(..) => {
                 return Ok(ProtobufConstant::Ident(self.next_full_ident()?));
-            },
-            _ => {},
+            }
+            _ => {}
         }
 
         Err(ParserError::ExpectConstant)
     }
 
     fn next_int_lit(&mut self) -> ParserResult<u64> {
-        self.next_token_check_map(|token| {
-            match token {
-                &Token::IntLit(i) => Ok(i),
-                _ => Err(ParserError::IncorrectInput),
-            }
+        self.next_token_check_map(|token| match token {
+            &Token::IntLit(i) => Ok(i),
+            _ => Err(ParserError::IncorrectInput),
         })
     }
 
@@ -1124,12 +1107,18 @@ impl<'a> Parser<'a> {
     // Import Statement
 
     // import = "import" [ "weak" | "public" ] strLit ";"
-    fn next_import_opt(&mut self) -> ParserResult<Option<String>> {
+    fn next_import_opt(&mut self) -> ParserResult<Option<Import>> {
         if self.next_ident_if_eq("import")? {
-            self.next_ident_if_in(&["weak", "public"])?;
-            let import_path = self.next_str_lit()?.decode_utf8()?;
+            let vis = if self.next_ident_if_eq("weak")? {
+                ImportVis::Weak
+            } else if self.next_ident_if_eq("public")? {
+                ImportVis::Public
+            } else {
+                ImportVis::Default
+            };
+            let path = self.next_str_lit()?.decode_utf8()?;
             self.next_symbol_expect_eq(';')?;
-            Ok(Some(import_path))
+            Ok(Some(Import { path, vis }))
         } else {
             Ok(None)
         }
@@ -1249,11 +1238,9 @@ impl<'a> Parser<'a> {
     }
 
     fn next_field_number(&mut self) -> ParserResult<i32> {
-        self.next_token_check_map(|token| {
-            match token {
-                &Token::IntLit(i) => i.to_i32(),
-                _ => Err(ParserError::IncorrectInput),
-            }
+        self.next_token_check_map(|token| match token {
+            &Token::IntLit(i) => i.to_i32(),
+            _ => Err(ParserError::IncorrectInput),
         })
     }
 
@@ -1299,15 +1286,23 @@ impl<'a> Parser<'a> {
                 Syntax::Proto3 => MessageBodyParseMode::MessageProto3,
             };
 
-            let MessageBody {
-                fields,
-                ..
-            } = self.next_message_body(mode)?;
+            let MessageBody { fields, .. } = self.next_message_body(mode)?;
+
+            let fields = fields
+                .into_iter()
+                .map(|fo| match fo {
+                    FieldOrOneOf::Field(f) => Ok(f),
+                    FieldOrOneOf::OneOf(_) => Err(ParserError::OneOfInGroup),
+                })
+                .collect::<Result<_, ParserError>>()?;
 
             Ok(Field {
-                name,
+                // The field name is a lowercased version of the type name
+                // (which has been verified to start with an uppercase letter).
+                // https://git.io/JvxAP
+                name: name.to_ascii_lowercase(),
                 rule,
-                typ: FieldType::Group(fields),
+                typ: FieldType::Group { name, fields },
                 number,
                 options: Vec::new(),
             })
@@ -1342,10 +1337,14 @@ impl<'a> Parser<'a> {
         if self.next_ident_if_eq("oneof")? {
             let name = self.next_ident()?.to_owned();
             let MessageBody { fields, .. } = self.next_message_body(MessageBodyParseMode::Oneof)?;
-            Ok(Some(OneOf {
-                name,
-                fields,
-            }))
+            let fields = fields
+                .into_iter()
+                .map(|fo| match fo {
+                    FieldOrOneOf::Field(f) => Ok(f),
+                    FieldOrOneOf::OneOf(_) => Err(ParserError::OneOfInOneOf),
+                })
+                .collect::<Result<_, ParserError>>()?;
+            Ok(Some(OneOf { name, fields }))
         } else {
             Ok(None)
         }
@@ -1413,17 +1412,16 @@ impl<'a> Parser<'a> {
     // fieldNames = fieldName { "," fieldName }
     fn next_reserved_opt(&mut self) -> ParserResult<Option<(Vec<FieldNumberRange>, Vec<String>)>> {
         if self.next_ident_if_eq("reserved")? {
-            let (ranges, names) =
-                if let &Token::StrLit(..) = self.lookahead_some()? {
-                    let mut names = Vec::new();
+            let (ranges, names) = if let &Token::StrLit(..) = self.lookahead_some()? {
+                let mut names = Vec::new();
+                names.push(self.next_str_lit()?.decode_utf8()?);
+                while self.next_symbol_if_eq(',')? {
                     names.push(self.next_str_lit()?.decode_utf8()?);
-                    while self.next_symbol_if_eq(',')? {
-                        names.push(self.next_str_lit()?.decode_utf8()?);
-                    }
-                    (Vec::new(), names)
-                } else {
-                    (self.next_ranges()?, Vec::new())
-                };
+                }
+                (Vec::new(), names)
+            } else {
+                (self.next_ranges()?, Vec::new())
+            };
 
             self.next_symbol_expect_eq(';')?;
 
@@ -1500,7 +1498,11 @@ impl<'a> Parser<'a> {
                 values.push(self.next_enum_field()?);
             }
             self.next_symbol_expect_eq('}')?;
-            Ok(Some(Enumeration { name, values, options }))
+            Ok(Some(Enumeration {
+                name,
+                values,
+                options,
+            }))
         } else {
             Ok(None)
         }
@@ -1529,7 +1531,7 @@ impl<'a> Parser<'a> {
                 }
 
                 if let Some(oneof) = self.next_oneof_opt()? {
-                    r.oneofs.push(oneof);
+                    r.fields.push(FieldOrOneOf::OneOf(oneof));
                     continue;
                 }
 
@@ -1550,7 +1552,6 @@ impl<'a> Parser<'a> {
                     r.enums.push(nested_enum);
                     continue;
                 }
-
             } else {
                 self.next_ident_if_eq_error("reserved")?;
                 self.next_ident_if_eq_error("oneof")?;
@@ -1569,7 +1570,7 @@ impl<'a> Parser<'a> {
                 self.next_ident_if_eq_error("option")?;
             }
 
-            r.fields.push(self.next_field(mode)?);
+            r.fields.push(FieldOrOneOf::Field(self.next_field(mode)?));
         }
 
         self.next_symbol_expect_eq('}')?;
@@ -1589,7 +1590,6 @@ impl<'a> Parser<'a> {
 
             let MessageBody {
                 fields,
-                oneofs,
                 reserved_nums,
                 reserved_names,
                 messages,
@@ -1600,7 +1600,6 @@ impl<'a> Parser<'a> {
             Ok(Some(Message {
                 name,
                 fields,
-                oneofs,
                 reserved_nums,
                 reserved_names,
                 messages,
@@ -1632,10 +1631,22 @@ impl<'a> Parser<'a> {
 
             let MessageBody { fields, .. } = self.next_message_body(mode)?;
 
-            let extensions = fields.into_iter().map(|field| {
-                let extendee = extendee.clone();
-                Extension { extendee, field }
-            }).collect();
+            // TODO: is oneof allowed in extend?
+            let fields: Vec<Field> = fields
+                .into_iter()
+                .map(|fo| match fo {
+                    FieldOrOneOf::Field(f) => Ok(f),
+                    FieldOrOneOf::OneOf(_) => Err(ParserError::OneOfInExtend),
+                })
+                .collect::<Result<_, ParserError>>()?;
+
+            let extensions = fields
+                .into_iter()
+                .map(|field| {
+                    let extendee = extendee.clone();
+                    Extension { extendee, field }
+                })
+                .collect();
 
             Ok(Some(extensions))
         } else {
@@ -1686,21 +1697,21 @@ impl<'a> Parser<'a> {
         let syntax = self.next_syntax()?.unwrap_or(Syntax::Proto2);
         self.syntax = syntax;
 
-        let mut import_paths = Vec::new();
-        let mut package = String::new();
+        let mut imports = Vec::new();
+        let mut package = None;
         let mut messages = Vec::new();
         let mut enums = Vec::new();
         let mut extensions = Vec::new();
         let mut options = Vec::new();
 
         while !self.syntax_eof()? {
-            if let Some(import_path) = self.next_import_opt()? {
-                import_paths.push(import_path);
+            if let Some(import) = self.next_import_opt()? {
+                imports.push(import);
                 continue;
             }
 
             if let Some(next_package) = self.next_package_opt()? {
-                package = next_package.to_owned();
+                package = Some(next_package);
                 continue;
             }
 
@@ -1736,7 +1747,7 @@ impl<'a> Parser<'a> {
         }
 
         Ok(FileDescriptor {
-            import_paths,
+            imports,
             package,
             syntax,
             messages,
@@ -1752,51 +1763,53 @@ mod test {
     use super::*;
 
     fn lex<P, R>(input: &str, parse_what: P) -> R
-        where P : FnOnce(&mut Lexer) -> ParserResult<R>
+    where
+        P: FnOnce(&mut Lexer) -> ParserResult<R>,
     {
         let mut lexer = Lexer {
             input,
             pos: 0,
             loc: Loc::start(),
         };
-        let r = parse_what(&mut lexer)
-            .expect(&format!("lexer failed at {}", lexer.loc));
+        let r = parse_what(&mut lexer).expect(&format!("lexer failed at {}", lexer.loc));
         assert!(lexer.eof(), "check eof failed at {}", lexer.loc);
         r
     }
 
     fn lex_opt<P, R>(input: &str, parse_what: P) -> R
-        where P : FnOnce(&mut Lexer) -> ParserResult<Option<R>>
+    where
+        P: FnOnce(&mut Lexer) -> ParserResult<Option<R>>,
     {
         let mut lexer = Lexer {
             input,
             pos: 0,
             loc: Loc::start(),
         };
-        let o = parse_what(&mut lexer)
-            .expect(&format!("lexer failed at {}", lexer.loc));
+        let o = parse_what(&mut lexer).expect(&format!("lexer failed at {}", lexer.loc));
         let r = o.expect(&format!("lexer returned none at {}", lexer.loc));
         assert!(lexer.eof(), "check eof failed at {}", lexer.loc);
         r
     }
 
     fn parse<P, R>(input: &str, parse_what: P) -> R
-        where P : FnOnce(&mut Parser) -> ParserResult<R>
+    where
+        P: FnOnce(&mut Parser) -> ParserResult<R>,
     {
         let mut parser = Parser::new(input);
-        let r = parse_what(&mut parser)
-            .expect(&format!("parse failed at {}", parser.loc()));
-        let eof = parser.syntax_eof().expect(&format!("check eof failed at {}", parser.loc()));
+        let r = parse_what(&mut parser).expect(&format!("parse failed at {}", parser.loc()));
+        let eof = parser
+            .syntax_eof()
+            .expect(&format!("check eof failed at {}", parser.loc()));
         assert!(eof, "{}", parser.loc());
         r
     }
 
     fn parse_opt<P, R>(input: &str, parse_what: P) -> R
-        where P : FnOnce(&mut Parser) -> ParserResult<Option<R>>
+    where
+        P: FnOnce(&mut Parser) -> ParserResult<Option<R>>,
     {
         let mut parser = Parser::new(input);
-        let o = parse_what(&mut parser)
-            .expect(&format!("parse failed at {}", parser.loc()));
+        let o = parse_what(&mut parser).expect(&format!("parse failed at {}", parser.loc()));
         let r = o.expect(&format!("parser returned none at {}", parser.loc()));
         assert!(parser.syntax_eof().unwrap());
         r
@@ -1807,7 +1820,6 @@ mod test {
         let msg = r#"10"#;
         let mess = lex_opt(msg, |p| p.next_int_lit_opt());
         assert_eq!(10, mess);
-
     }
 
     #[test]
@@ -1815,7 +1827,6 @@ mod test {
         let msg = r#"12.3"#;
         let mess = lex(msg, |p| p.next_token_inner());
         assert_eq!(Token::FloatLit(12.3), mess);
-
     }
 
     #[test]
@@ -1829,7 +1840,12 @@ mod test {
     fn test_str_lit() {
         let msg = r#"  "a\nb"  "#;
         let mess = parse(msg, |p| p.next_str_lit());
-        assert_eq!(StrLit { escaped: r#"a\nb"#.to_owned() }, mess);
+        assert_eq!(
+            StrLit {
+                escaped: r#"a\nb"#.to_owned()
+            },
+            mess
+        );
     }
 
     #[test]
@@ -1897,25 +1913,6 @@ mod test {
         parse_opt(msg, |p| p.next_option_opt());
     }
 
-    #[test]
-    fn test_import() {
-        let msg = r#"syntax = "proto3";
-
-    import "test_import_nested_imported_pb.proto";
-
-    message ContainsImportedNested {
-        ContainerForNested.NestedMessage m = 1;
-        ContainerForNested.NestedEnum e = 2;
-    }
-    "#;
-        let desc = parse(msg, |p| p.next_proto());
-
-        assert_eq!(
-            vec!["test_import_nested_imported_pb.proto"],
-            desc.import_paths
-        );
-    }
-
     #[test]
     fn test_package() {
         let msg = r#"
@@ -1927,7 +1924,18 @@ mod test {
     }
     "#;
         let desc = parse(msg, |p| p.next_proto());
-        assert_eq!("foo.bar".to_string(), desc.package);
+        assert_eq!(Some("foo.bar".to_string()), desc.package);
+    }
+
+    #[test]
+    fn test_no_package() {
+        let msg = r#"
+    message A {
+        optional int32 a = 1;
+    }
+    "#;
+        let desc = parse(msg, |p| p.next_proto());
+        assert_eq!(None, desc.package);
     }
 
     #[test]
@@ -1954,7 +1962,7 @@ mod test {
 
         let mess = parse_opt(msg, |p| p.next_message_opt());
         assert_eq!(1, mess.fields.len());
-        match mess.fields[0].typ {
+        match mess.regular_fields_for_test()[0].typ {
             FieldType::Map(ref f) => match &**f {
                 &(FieldType::String, FieldType::Int32) => (),
                 ref f => panic!("Expecting Map<String, Int32> found {:?}", f),
@@ -1977,8 +1985,8 @@ mod test {
     }"#;
 
         let mess = parse_opt(msg, |p| p.next_message_opt());
-        assert_eq!(1, mess.oneofs.len());
-        assert_eq!(3, mess.oneofs[0].fields.len());
+        assert_eq!(1, mess.oneofs_for_test().len());
+        assert_eq!(3, mess.oneofs_for_test()[0].fields.len());
     }
 
     #[test]
@@ -1991,12 +1999,15 @@ mod test {
     }"#;
 
         let mess = parse_opt(msg, |p| p.next_message_opt());
-        assert_eq!(vec![
-            FieldNumberRange { from: 4, to: 4 },
-            FieldNumberRange { from: 15, to: 15 },
-            FieldNumberRange { from: 17, to: 20 },
-            FieldNumberRange { from: 30, to: 30 }],
-            mess.reserved_nums);
+        assert_eq!(
+            vec![
+                FieldNumberRange { from: 4, to: 4 },
+                FieldNumberRange { from: 15, to: 15 },
+                FieldNumberRange { from: 17, to: 20 },
+                FieldNumberRange { from: 30, to: 30 }
+            ],
+            mess.reserved_nums
+        );
         assert_eq!(
             vec!["foo".to_string(), "bar".to_string()],
             mess.reserved_names
@@ -2011,8 +2022,11 @@ mod test {
         }"#;
 
         let mess = parse_opt(msg, |p| p.next_message_opt());
-        assert_eq!("default", mess.fields[0].options[0].name);
-        assert_eq!("17", mess.fields[0].options[0].value.format());
+        assert_eq!("default", mess.regular_fields_for_test()[0].options[0].name);
+        assert_eq!(
+            "17",
+            mess.regular_fields_for_test()[0].options[0].value.format()
+        );
     }
 
     #[test]
@@ -2024,7 +2038,8 @@ mod test {
         let mess = parse_opt(msg, |p| p.next_message_opt());
         assert_eq!(
             r#""ab\nc d\"g\'h\0\"z""#,
-            mess.fields[0].options[0].value.format());
+            mess.regular_fields_for_test()[0].options[0].value.format()
+        );
     }
 
     #[test]
@@ -2036,7 +2051,8 @@ mod test {
         let mess = parse_opt(msg, |p| p.next_message_opt());
         assert_eq!(
             r#""ab\nc d\xfeE\"g\'h\0\"z""#,
-            mess.fields[0].options[0].value.format());
+            mess.regular_fields_for_test()[0].options[0].value.format()
+        );
     }
 
     #[test]
@@ -2053,14 +2069,14 @@ mod test {
         }"#;
         let mess = parse_opt(msg, |p| p.next_message_opt());
 
-        assert_eq!("Identifier", mess.fields[1].name);
-        if let FieldType::Group(ref group_fields) = mess.fields[1].typ {
-            assert_eq!(2, group_fields.len());
+        assert_eq!("identifier", mess.regular_fields_for_test()[1].name);
+        if let FieldType::Group { name, fields } = &mess.regular_fields_for_test()[1].typ {
+            assert_eq!(2, fields.len());
         } else {
             panic!("expecting group");
         }
 
-        assert_eq!("bbb", mess.fields[2].name);
+        assert_eq!("bbb", mess.regular_fields_for_test()[2].name);
     }
 
     #[test]
